# -*- coding: utf-8 -*-
"""
@author: jbechtloff
Vorgabe: tsA, tsE, Timedelta

Zusammenführen von  sim_sig_design_03.py &
                    sim_mdb_struc_03.py
Generieren + Realtime                    
"""

import numpy as np
import pandas as pd
from PT1 import PT1

import matplotlib.pyplot as plt
plt.close('all')


class clProzess:
    def __init__(self,  T, T1, Kdrift, Tdrift, Kw):
        self.pt1a = PT1(T,T1)
        self.pt1b = PT1(T,T1)
        self.T = T
        self.Kdrift = Kdrift
        self.Tdrift = Tdrift
        self.Kw = Kw
        self.cnt = 0

    def simProz(self):
        k = self.cnt
        u_1 = np.sin(4*k/100*np.pi)
        if u_1 <0: u_1=0
        if u_1 > 0.5: u_1 = 0.5
        u_1 = u_1*2.0

        # Amplituden-Drift-Simu: 1.0 < u2 < 1.1 (MUL)
        a_Drift = 1.0 + self.Kdrift * (np.sin(4*k*self.T/self.Tdrift*np.pi)+1.0)/2.0
        errFl = 0
        if a_Drift > 1.0+19*self.Kdrift/20: errFl = 1.0
        u_2 = self.pt1b.PT1rek(self.pt1a.PT1rek(u_1*a_Drift))  # PT2-Verschleiß
        w = np.random.randn()*self.Kw

        self.cnt +=1
        return(u_1, u_2, a_Drift, w, errFl)

    def write_(self, ts):
        [u_1, u_2, a_Drift, wi, err_Flg] = self.simProz()

        data = {
            'ts': ts,
            'cnt': self.cnt,
            'u': u_2 + wi,
            'err': err_Flg
        }
        # print("%s" % (self.cnt))
        # print(data)
        return data


if __name__ == "__main__":

    Prozess = clProzess(T=1,T1=4,
                    Kdrift=0.1, 
                    Tdrift=2000, 
                    Kw=0.01)
    
    Tscal = {"s":1,"m":60,"h":60*60, "d":60*60*24}
    dT = 1
    dT_unit = "s"

    TsimSpan = 2
    TsimSpan_unit = "m"

    n = int(np.trunc(TsimSpan * Tscal[TsimSpan_unit] / (dT*Tscal[dT_unit])))

    vU = []
    vTS = []
    vErr = []

    # tsA = pd.Timestamp(2021,11, 20,  0,0,0)
    # tsE = pd.Timestamp(2021,11, 21,  10,0,0)
    # tsDeltaAE = tsE - tsA

    tsE = pd.Timestamp.now()        # now
    tsA = tsE - pd.Timedelta(TsimSpan,TsimSpan_unit) # Vorlauf

    ts = tsA
    while ts <= tsE: 
        data = Prozess.write_(ts)
        ts = ts + pd.Timedelta(dT,dT_unit)
        vU.append(data["u"])
        # vTS[data["cnt"]] = ts
        vTS.append(ts)
        vErr.append(data["err"])



    # Plot-Ausgabe -----------------------------------------

fig = plt.figure(figsize=(5, 4))
# df_F.plot(x="ts",y="u")

# matplotlib auf pd.Datetime einstellen

plt.plot(vTS,vU)
plt.plot(vTS,vErr,color='#FF0000')
plt.show()
    
